@@ -11,6 +11,8 @@ types = [np.int8, np.uint8, np.int16,
          np.float32, np.float64]
 
 
+np.mod(1., 1)  # Silence fmod bug on win-amd64. See #1408 and #1238.
+
 class Test_measurements_stats(TestCase):
     """ndimage.measurements._stats() is a utility function used by other functions."""
 
@@ -522,6 +524,41 @@ def test_maximum05():
     x = np.array([-3,-2,-1])
     assert_equal(ndimage.maximum(x),-1)
 
+def test_median01():
+    "median 1"
+    a = np.array([[1, 2, 0, 1],
+                  [5, 3, 0, 4],
+                  [0, 0, 0, 7],
+                  [9, 3, 0, 0]])
+    labels = np.array([[1, 1, 0, 2],
+                       [1, 1, 0, 2],
+                       [0, 0, 0, 2],
+                       [3, 3, 0, 0]])
+    output = ndimage.median(a, labels=labels, index=[1, 2, 3])
+    assert_array_almost_equal(output, [2.5, 4.0, 6.0])
+
+def test_median02():
+    "median 2"
+    a = np.array([[1, 2, 0, 1],
+                  [5, 3, 0, 4],
+                  [0, 0, 0, 7],
+                  [9, 3, 0, 0]])
+    output = ndimage.median(a)
+    assert_almost_equal(output, 1.0)
+
+def test_median03():
+    "median 3"
+    a = np.array([[1, 2, 0, 1],
+                  [5, 3, 0, 4],
+                  [0, 0, 0, 7],
+                  [9, 3, 0, 0]])
+    labels = np.array([[1, 1, 0, 2],
+                       [1, 1, 0, 2],
+                       [0, 0, 0, 2],
+                       [3, 3, 0, 0]])
+    output = ndimage.median(a, labels=labels)
+    assert_almost_equal(output, 3.0)
+
 def test_variance01():
     "variance 1"
     olderr = np.seterr(all='ignore')
@@ -759,6 +796,18 @@ def test_maximum_position06():
         assert_equal(output[0], (0, 0))
         assert_equal(output[1], (1, 1))
 
+def test_maximum_position07():
+    "maximum position 7 - float labels"
+    labels = np.array([1.0, 2.5, 0.0, 4.5])
+    for type in types:
+        input = np.array([[5, 4, 2, 5],
+                          [3, 7, 8, 2],
+                          [1, 5, 1, 1]], type)
+        output = ndimage.maximum_position(input, labels,
+                                          [1.0, 4.5])
+        assert_equal(output[0], (0, 0))
+        assert_equal(output[1], (0, 3))
+
 def test_extrema01():
     "extrema 1"
     labels = np.array([1, 0], bool)
