@@ -181,6 +181,9 @@ def fmin(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None, maxfun=None,
     """
     Minimize a function using the downhill simplex algorithm.
 
+    This algorithm only uses function values, not derivatives or second
+    derivatives.
+
     Parameters
     ----------
     func : callable func(x,*args)
@@ -231,6 +234,24 @@ def fmin(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None, maxfun=None,
     Uses a Nelder-Mead simplex algorithm to find the minimum of function of
     one or more variables.
 
+    This algorithm has a long history of successful use in applications.
+    But it will usually be slower than an algorithm that uses first or
+    second derivative information. In practice it can have poor
+    performance in high-dimensional problems and is not robust to
+    minimizing complicated functions. Additionally, there currently is no
+    complete theory describing when the algorithm will successfully
+    converge to the minimum, or how fast it will if it does.
+
+    References
+    ----------
+    Nelder, J.A. and Mead, R. (1965), "A simplex method for function
+    minimization", The Computer Journal, 7, pp. 308-313
+    Wright, M.H. (1996), "Direct Search Methods: Once Scorned, Now
+    Respectable", in Numerical Analysis 1995, Proceedings of the
+    1995 Dundee Biennial Conference in Numerical Analysis, D.F.
+    Griffiths and G.A. Watson (Eds.), Addison Wesley Longman,
+    Harlow, UK, pp. 191-208.
+
     """
     fcalls, func = wrap_function(func, args)
     x0 = asfarray(x0).flatten()
@@ -379,6 +400,31 @@ def approx_fprime(xk,f,epsilon,*args):
     return grad
 
 def check_grad(func, grad, x0, *args):
+    """Check the correctness of a gradient function
+    by comparing it against a finite-difference approximation
+    of the gradient.
+
+    Parameters
+    ----------
+    func: callable func(x0,*args)
+        Function whose derivative is to be checked
+    grad: callable grad(x0, *args)
+        Gradient of func
+    x0: ndarray
+        Points to check grad against finite difference
+        approximation of grad using func.
+    args: optional
+        Extra arguments passed to func and grad
+
+    Returns
+    -------
+    err: float
+        The square root of the sum of squares (i.e. the 2-norm)
+        of the difference between grad(x0, *args) and the
+        finite difference approximation of grad using func at the
+        points x0.
+
+    """
     return sqrt(sum((grad(x0,*args)-approx_fprime(x0,func,_epsilon,*args))**2))
 
 def approx_fhess_p(x0,p,fprime,epsilon,*args):
@@ -449,8 +495,11 @@ def fmin_bfgs(f, x0, fprime=None, args=(), gtol=1e-5, norm=Inf,
     -----
     Optimize the function, f, whose gradient is given by fprime
     using the quasi-Newton method of Broyden, Fletcher, Goldfarb,
-    and Shanno (BFGS) See Wright, and Nocedal 'Numerical
-    Optimization', 1999, pg. 198.
+    and Shanno (BFGS)
+
+    References
+    ----------
+    Wright, and Nocedal 'Numerical Optimization', 1999, pg. 198.
 
     """
     x0 = asarray(x0).flatten()
@@ -510,6 +559,12 @@ def fmin_bfgs(f, x0, fprime=None, args=(), gtol=1e-5, norm=Inf,
         if (gnorm <= gtol):
             break
 
+        if not numpy.isfinite(old_fval):
+            # We correctly found +-Inf as optimal value, or something went
+            # wrong.
+            warnflag = 2
+            break
+
         try: # this was handled in numeric, let it remaines for more safety
             rhok = 1.0 / (numpy.dot(yk,sk))
         except ZeroDivisionError:
@@ -720,7 +775,8 @@ def fmin_cg(f, x0, fprime=None, args=(), gtol=1e-5, norm=Inf, epsilon=_epsilon,
 def fmin_ncg(f, x0, fprime, fhess_p=None, fhess=None, args=(), avextol=1e-5,
              epsilon=_epsilon, maxiter=None, full_output=0, disp=1, retall=0,
              callback=None):
-    """Minimize a function using the Newton-CG method.
+    """Unconstrained minimization of a function using the Newton-CG method.
+
 
     Parameters
     ----------
@@ -786,8 +842,21 @@ def fmin_ncg(f, x0, fprime, fhess_p=None, fhess=None, args=(), avextol=1e-5,
     approximated using finite differences on `fprime`. `fhess_p`
     must compute the hessian times an arbitrary vector. If it is not
     given, finite-differences on `fprime` are used to compute
-    it. See Wright & Nocedal, 'Numerical Optimization', 1999,
-    pg. 140.
+    it.
+
+    Newton-CG methods are also called truncated Newton methods. This
+    function differs from scipy.optimize.fmin_tnc because
+
+    1. scipy.optimize.fmin_ncg is written purely in python using numpy
+        and scipy while scipy.optimize.fmin_tnc calls a C function.
+    2. scipy.optimize.fmin_ncg is only for unconstrained minimization
+        while scipy.optimize.fmin_tnc is for unconstrained minimization
+        or box constrained minimization. (Box constraints give
+        lower and upper bounds for each variable seperately.)
+
+    References
+    ----------
+    Wright & Nocedal, 'Numerical Optimization', 1999, pg. 140.
 
     """
     x0 = asarray(x0).flatten()
@@ -1094,11 +1163,13 @@ class Brent:
             xa,xb,xc = brack
             if (xa > xc):  # swap so xa < xc can be assumed
                 dum = xa; xa=xc; xc=dum
-            assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
+            if not ((xa < xb) and (xb < xc)):
+                raise ValueError("Not a bracketing interval.")
             fa = func(*((xa,)+args))
             fb = func(*((xb,)+args))
             fc = func(*((xc,)+args))
-            assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
+            if not ((fb<fa) and (fb < fc)):
+                raise ValueError("Not a bracketing interval.")
             funcalls = 3
         else:
             raise ValueError("Bracketing interval must be " \
@@ -1278,11 +1349,13 @@ def golden(func, args=(), brack=None, tol=_epsilon, full_output=0):
         xa,xb,xc = brack
         if (xa > xc):  # swap so xa < xc can be assumed
             dum = xa; xa=xc; xc=dum
-        assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
+        if not ((xa < xb) and (xb < xc)):
+            raise ValueError("Not a bracketing interval.")
         fa = func(*((xa,)+args))
         fb = func(*((xb,)+args))
         fc = func(*((xc,)+args))
-        assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
+        if not ((fb<fa) and (fb < fc)):
+            raise ValueError("Not a bracketing interval.")
         funcalls = 3
     else:
         raise ValueError("Bracketing interval must be length 2 or 3 sequence.")
@@ -1424,7 +1497,8 @@ def fmin_powell(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None,
                 maxfun=None, full_output=0, disp=1, retall=0, callback=None,
                 direc=None):
     """
-    Minimize a function using modified Powell's method.
+    Minimize a function using modified Powell's method. This method
+    only uses function values, not derivatives.
 
     Parameters
     ----------
@@ -1481,7 +1555,35 @@ def fmin_powell(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None,
     Notes
     -----
     Uses a modification of Powell's method to find the minimum of
-    a function of N variables.
+    a function of N variables. Powell's method is a conjugate
+    direction method.
+
+    The algorithm has two loops. The outer loop
+    merely iterates over the inner loop. The inner loop minimizes
+    over each current direction in the direction set. At the end
+    of the inner loop, if certain conditions are met, the direction
+    that gave the largest decrease is dropped and replaced with
+    the difference between the current estiamted x and the estimated
+    x from the beginning of the inner-loop.
+
+    The technical conditions for replacing the direction of greatest
+    increase amount to checking that
+
+    1. No further gain can be made along the direction of greatest increase
+       from that iteration.
+    2. The direction of greatest increase accounted for a large sufficient
+       fraction of the decrease in the function value from that iteration of
+       the inner loop.
+
+
+    References
+    ----------
+    Powell M.J.D. (1964) An efficient method for finding the minimum of a
+    function of several variables without calculating derivatives,
+    Computer Journal, 7 (2):155-162.
+
+    Press W., Teukolsky S.A., Vetterling W.T., and Flannery B.P.:
+    Numerical Recipes (any edition), Cambridge University Press
 
     """
     # we need to use a mutable object here that we can update in the
@@ -1608,6 +1710,11 @@ def brute(func, ranges, args=(), Ns=20, full_output=0, finish=fmin):
         Default number of samples, if those are not provided.
     full_output : bool
         If True, return the evaluation grid.
+    finish : callable, optional
+        An optimization function that is called with the result of brute force
+        minimization as initial guess.  `finish` should take the initial guess
+        as positional argument, and take take `args`, `full_output` and `disp`
+        as keyword arguments.  See Notes for more details.
 
     Returns
     -------
@@ -1623,8 +1730,10 @@ def brute(func, ranges, args=(), Ns=20, full_output=0, finish=fmin):
 
     Notes
     -----
-    Find the minimum of a function evaluated on a grid given by
-    the tuple ranges.
+    The range is respected by the brute force minimization, but if the `finish`
+    keyword specifies another optimization function (including the default
+    `fmin`), the returned value may still be (just) outside the range.  In
+    order to ensure the range is specified, use ``finish=None``.
 
     """
     N = len(ranges)
