@@ -34,6 +34,9 @@ value of the function, and whose second argument is the gradient of the function
 from scipy.optimize import moduleTNC
 from numpy import asarray, inf, array
 
+__all__ = ['fmin_tnc']
+
+
 MSG_NONE = 0 # No messages
 MSG_ITER = 1 # One line per iteration
 MSG_INFO = 2 # Informational messages
@@ -85,28 +88,38 @@ def fmin_tnc(func, x0, fprime=None, args=(), approx_grad=0,
              rescale=-1, disp=None):
     """
     Minimize a function with variables subject to bounds, using
-    gradient information.
+    gradient information in a truncated Newton algorithm. This
+    method wraps a C implementation of the algorithm.
 
     Parameters
     ----------
     func : callable ``func(x, *args)``
-        Function to minimize.  Should return f and g, where f is
+        Function to minimize.  Must do one of
+        1. Return f and g, where f is
         the value of the function and g its gradient (a list of
-        floats).  If the function returns None, the minimization
+        floats).
+        2. Return the function value but supply gradient function
+        seperately as fprime
+        3. Return the function value and set approx_grad=True.
+        If the function returns None, the minimization
         is aborted.
     x0 : list of floats
         Initial estimate of minimum.
     fprime : callable ``fprime(x, *args)``
-        Gradient of func. If None, then func must return the
-        function value and the gradient (``f,g = func(x, *args)``).
+        Gradient of func. If None, then either func must return the
+        function value and the gradient (``f,g = func(x, *args)``)
+        or approx_grad must be True.
     args : tuple
         Arguments to pass to function.
     approx_grad : bool
         If true, approximate the gradient numerically.
     bounds : list
-        (min, max) pairs for each element in x, defining the
+        (min, max) pairs for each element in x0, defining the
         bounds on that parameter. Use None or +/-inf for one of
         min or max when there is no bound in that direction.
+    epsilon: float
+        Used if approx_grad is True. The stepsize in a finite
+        difference approximation for fprime.
     scale : list of floats
         Scaling factors to apply to each variable.  If None, the
         factors are up-low for interval bounded variables and
@@ -168,6 +181,42 @@ def fmin_tnc(func, x0, fprime=None, args=(), approx_grad=0,
     rc : int
         Return code as defined in the RCSTRINGS dict.
 
+
+    Notes
+    -----
+    The underlying algorithm is truncated Newton, also called
+    Newton Conjugate-Gradient. This method differs from
+    scipy.optimize.fmin_ncg in that
+
+    1. It wraps a C implementation of the algorithm
+    2. It allows each variable to be given an upper and lower bound.
+
+
+    The algorithm incoporates the bound constraints by determining
+    the descent direction as in an unconstrained truncated Newton,
+    but never taking a step-size large enough to leave the space
+    of feasible x's. The algorithm keeps track of a set of
+    currently active constraints, and ignores them when computing
+    the minimum allowable step size. (The x's associated with the
+    active constraint are kept fixed.) If the maximum allowable
+    step size is zero then a new constraint is added. At the end
+    of each iteration one of the constraints may be deemed no
+    longer active and removed. A constraint is considered
+    no longer active is if it is currently active
+    but the gradient for that variable points inward from the
+    constraint. The specific constraint removed is the one
+    associated with the variable of largest index whose
+    constraint is no longer active.
+
+
+    References
+    ----------
+    Wright S., Nocedal J. (2006), 'Numerical Optimization'
+
+    Nash S.G. (1984), "Newton-Type Minimization Via the Lanczos Method",
+    SIAM Journal of Numerical Analysis 21, pp. 770-778
+
+
     """
     x0 = asarray(x0, dtype=float).tolist()
     n = len(x0)
@@ -178,7 +227,7 @@ def fmin_tnc(func, x0, fprime=None, args=(), approx_grad=0,
         raise ValueError('length of x0 != length of bounds')
 
     if disp is not None:
-        messages = {0:MSG_NONE, 1:MSG_ITER, 2:MSG_INFO, 3:MSG_VERS, 
+        messages = {0:MSG_NONE, 1:MSG_ITER, 2:MSG_INFO, 3:MSG_VERS,
                     4:MSG_EXIT, 5:MSG_ALL}.get(disp, MSG_ALL)
 
     if approx_grad:
