@@ -1,7 +1,6 @@
 # Author: Travis Oliphant
 # 1999 -- 2002
 
-import warnings
 
 import sigtools
 from scipy import linalg
@@ -9,34 +8,44 @@ from scipy.fftpack import fft, ifft, ifftshift, fft2, ifft2, fftn, \
         ifftn, fftfreq
 from numpy import polyadd, polymul, polydiv, polysub, roots, \
         poly, polyval, polyder, cast, asarray, isscalar, atleast_1d, \
-        ones, real, real_if_close, zeros, array, arange, where, rank, \
+        ones, real_if_close, zeros, array, arange, where, rank, \
         newaxis, product, ravel, sum, r_, iscomplexobj, take, \
         argsort, allclose, expand_dims, unique, prod, sort, reshape, \
-        transpose, dot, any, mean, flipud, ndarray
+        transpose, dot, mean, ndarray, atleast_2d
 import numpy as np
 from scipy.misc import factorial
 from windows import get_window
+from _arraytools import axis_slice, axis_reverse, odd_ext, even_ext, const_ext
 
-_modedict = {'valid':0, 'same':1, 'full':2}
+__all__ = ['correlate', 'fftconvolve', 'convolve', 'convolve2d', 'correlate2d',
+           'order_filter', 'medfilt', 'medfilt2d', 'wiener', 'lfilter',
+           'lfiltic', 'deconvolve', 'hilbert', 'hilbert2', 'cmplx_sort',
+           'unique_roots', 'invres', 'invresz', 'residue', 'residuez',
+           'resample', 'detrend', 'lfilter_zi', 'filtfilt', 'decimate']
+
+
+_modedict = {'valid': 0, 'same': 1, 'full': 2}
+
+_boundarydict = {'fill': 0, 'pad': 0, 'wrap': 2, 'circular': 2, 'symm': 1,
+                 'symmetric': 1, 'reflect': 4}
 
-_boundarydict = {'fill':0, 'pad':0, 'wrap':2, 'circular':2, 'symm':1,
-                 'symmetric':1, 'reflect':4}
 
 def _valfrommode(mode):
     try:
         val = _modedict[mode]
     except KeyError:
-        if mode not in [0,1,2]:
+        if mode not in [0, 1, 2]:
             raise ValueError("Acceptable mode flags are 'valid' (0),"
                     " 'same' (1), or 'full' (2).")
         val = mode
     return val
 
+
 def _bvalfromboundary(boundary):
     try:
         val = _boundarydict[boundary] << 2
     except KeyError:
-        if val not in [0,1,2] :
+        if val not in [0, 1, 2]:
             raise ValueError("Acceptable boundary flags are 'fill', 'wrap'"
                     " (or 'circular'), \n  and 'symm' (or 'symmetric').")
         val = boundary << 2
@@ -60,7 +69,7 @@ def correlate(in1, in2, mode='full'):
         a string indicating the size of the output:
             - 'valid': the output consists only of those elements that do not
             rely on the zero-padding.
-            - 'same': the output is the same size as the largest input centered
+            - 'same': the output is the same size as ``in1`` centered
               with respect to the 'full' output.
             - 'full': the output is the full discrete linear cross-correlation
               of the inputs. (Default)
@@ -110,6 +119,7 @@ def correlate(in1, in2, mode='full'):
 
     return z
 
+
 def _centered(arr, newsize):
     # Return the center newsize portion of the array.
     newsize = asarray(newsize)
@@ -119,6 +129,7 @@ def _centered(arr, newsize):
     myslice = [slice(startind[k], endind[k]) for k in range(len(endind))]
     return arr[tuple(myslice)]
 
+
 def fftconvolve(in1, in2, mode="full"):
     """Convolve two N-dimensional arrays using FFT. See convolve.
 
@@ -127,12 +138,12 @@ def fftconvolve(in1, in2, mode="full"):
     s2 = array(in2.shape)
     complex_result = (np.issubdtype(in1.dtype, np.complex) or
                       np.issubdtype(in2.dtype, np.complex))
-    size = s1+s2-1
+    size = s1 + s2 - 1
 
     # Always use 2**n-sized FFT
-    fsize = 2**np.ceil(np.log2(size))
-    IN1 = fftn(in1,fsize)
-    IN1 *= fftn(in2,fsize)
+    fsize = 2 ** np.ceil(np.log2(size))
+    IN1 = fftn(in1, fsize)
+    IN1 *= fftn(in2, fsize)
     fslice = tuple([slice(0, int(sz)) for sz in size])
     ret = ifftn(IN1)[fslice].copy()
     del IN1
@@ -141,13 +152,13 @@ def fftconvolve(in1, in2, mode="full"):
     if mode == "full":
         return ret
     elif mode == "same":
-        if product(s1,axis=0) > product(s2,axis=0):
+        if product(s1, axis=0) > product(s2, axis=0):
             osize = s1
         else:
             osize = s2
-        return _centered(ret,osize)
+        return _centered(ret, osize)
     elif mode == "valid":
-        return _centered(ret,abs(s2-s1)+1)
+        return _centered(ret, abs(s2 - s1) + 1)
 
 
 def convolve(in1, in2, mode='full'):
@@ -168,7 +179,7 @@ def convolve(in1, in2, mode='full'):
         ``valid`` : the output consists only of those elements that do not
            rely on the zero-padding.
 
-        ``same`` : the output is the same size as the largest input centered
+        ``same`` : the output is the same size as ``in1`` centered
            with respect to the 'full' output.
 
         ``full`` : the output is the full discrete linear cross-correlation
@@ -186,11 +197,11 @@ def convolve(in1, in2, mode='full'):
     kernel = asarray(in2)
 
     if rank(volume) == rank(kernel) == 0:
-        return volume*kernel
+        return volume * kernel
     elif not volume.ndim == kernel.ndim:
         raise ValueError("in1 and in2 should have the same rank")
 
-    slice_obj = [slice(None,None,-1)]*len(kernel.shape)
+    slice_obj = [slice(None, None, -1)] * len(kernel.shape)
 
     if mode == 'valid':
         for d1, d2 in zip(volume.shape, kernel.shape):
@@ -203,6 +214,7 @@ def convolve(in1, in2, mode='full'):
     else:
         return correlate(volume, kernel[slice_obj], mode)
 
+
 def order_filter(a, domain, rank):
     """
     Perform an order filter on an N-dimensional array.
@@ -303,9 +315,9 @@ def medfilt(volume, kernel_size=None):
 
     domain = ones(kernel_size)
 
-    numels = product(kernel_size,axis=0)
-    order = int(numels/2)
-    return sigtools._order_filterND(volume,domain,order)
+    numels = product(kernel_size, axis=0)
+    order = int(numels / 2)
+    return sigtools._order_filterND(volume, domain, order)
 
 
 def wiener(im, mysize=None, noise=None):
@@ -336,20 +348,21 @@ def wiener(im, mysize=None, noise=None):
     im = asarray(im)
     if mysize is None:
         mysize = [3] * len(im.shape)
-    mysize = asarray(mysize);
+    mysize = asarray(mysize)
 
     # Estimate the local mean
-    lMean = correlate(im,ones(mysize), 'same') / product(mysize,axis=0)
+    lMean = correlate(im, ones(mysize), 'same') / product(mysize, axis=0)
 
     # Estimate the local variance
-    lVar = correlate(im**2,ones(mysize), 'same') / product(mysize,axis=0) - lMean**2
+    lVar = (correlate(im ** 2, ones(mysize), 'same') / product(mysize, axis=0)
+            - lMean ** 2)
 
     # Estimate the noise power if needed.
-    if noise==None:
-        noise = mean(ravel(lVar),axis=0)
+    if noise == None:
+        noise = mean(ravel(lVar), axis=0)
 
     res = (im - lMean)
-    res *= (1-noise / lVar)
+    res *= (1 - noise / lVar)
     res += lMean
     out = where(lVar < noise, lMean, res)
 
@@ -372,7 +385,7 @@ def convolve2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
         ``valid`` : the output consists only of those elements that do not
            rely on the zero-padding.
 
-        ``same`` : the output is the same size as the largest input centered
+        ``same`` : the output is the same size as ``in1`` centered
            with respect to the 'full' output.
 
         ``full`` : the output is the full discrete linear cross-correlation
@@ -405,7 +418,8 @@ def convolve2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     val = _valfrommode(mode)
     bval = _bvalfromboundary(boundary)
 
-    return sigtools._convolve2d(in1,in2,1,val,bval,fillvalue)
+    return sigtools._convolve2d(in1, in2, 1, val, bval, fillvalue)
+
 
 def correlate2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     """Cross-correlate two 2-dimensional arrays.
@@ -423,7 +437,7 @@ def correlate2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
         ``valid`` : the output consists only of those elements that do not
            rely on the zero-padding.
 
-        ``same`` : the output is the same size as the largest input centered
+        ``same`` : the output is the same size as ``in1`` centered
            with respect to the 'full' output.
 
         ``full`` : the output is the full discrete linear cross-correlation
@@ -449,7 +463,8 @@ def correlate2d(in1, in2, mode='full', boundary='fill', fillvalue=0):
     val = _valfrommode(mode)
     bval = _bvalfromboundary(boundary)
 
-    return sigtools._convolve2d(in1, in2, 0,val,bval,fillvalue)
+    return sigtools._convolve2d(in1, in2, 0, val, bval, fillvalue)
+
 
 def medfilt2d(input, kernel_size=3):
     """
@@ -491,7 +506,6 @@ def medfilt2d(input, kernel_size=3):
     return sigtools._medfilt2d(image, kernel_size)
 
 
-
 def lfilter(b, a, x, axis=-1, zi=None):
     """
     Filter data along one-dimension with an IIR or FIR filter.
@@ -566,7 +580,8 @@ def lfilter(b, a, x, axis=-1, zi=None):
     else:
         return sigtools._linear_filter(b, a, x, axis, zi)
 
-def lfiltic(b,a,y,x=None):
+
+def lfiltic(b, a, y, x=None):
     """
     Construct initial conditions for lfilter
 
@@ -589,30 +604,31 @@ def lfiltic(b,a,y,x=None):
       zi = {z_0[-1], z_1[-1], ..., z_K-1[-1]}  where K=max(M,N).
 
     """
-    N = np.size(a)-1
-    M = np.size(b)-1
-    K = max(M,N)
+    N = np.size(a) - 1
+    M = np.size(b) - 1
+    K = max(M, N)
     y = asarray(y)
-    zi = zeros(K,y.dtype.char)
+    zi = zeros(K, y.dtype.char)
     if x is None:
-        x = zeros(M,y.dtype.char)
+        x = zeros(M, y.dtype.char)
     else:
         x = asarray(x)
         L = np.size(x)
         if L < M:
-            x = r_[x,zeros(M-L)]
+            x = r_[x, zeros(M - L)]
     L = np.size(y)
     if L < N:
-        y = r_[y,zeros(N-L)]
+        y = r_[y, zeros(N - L)]
 
     for m in range(M):
-        zi[m] = sum(b[m+1:]*x[:M-m],axis=0)
+        zi[m] = sum(b[m + 1:] * x[:M - m], axis=0)
 
     for m in range(N):
-        zi[m] -= sum(a[m+1:]*y[:N-m],axis=0)
+        zi[m] -= sum(a[m + 1:] * y[:N - m], axis=0)
 
     return zi
 
+
 def deconvolve(signal, divisor):
     """Deconvolves divisor out of signal.
 
@@ -622,17 +638,16 @@ def deconvolve(signal, divisor):
     N = len(num)
     D = len(den)
     if D > N:
-        quot = [];
-        rem = num;
+        quot = []
+        rem = num
     else:
-        input = ones(N-D+1, float)
+        input = ones(N - D + 1, float)
         input[1:] = 0
         quot = lfilter(num, den, input)
         rem = num - convolve(den, quot, mode='full')
     return quot, rem
 
 
-
 def hilbert(x, N=None, axis=-1):
     """
     Compute the analytic signal.
@@ -671,39 +686,39 @@ def hilbert(x, N=None, axis=-1):
 
     """
     x = asarray(x)
+    if iscomplexobj(x):
+        raise ValueError("x must be real.")
     if N is None:
         N = x.shape[axis]
-    if N <=0:
+    if N <= 0:
         raise ValueError("N must be positive.")
-    if iscomplexobj(x):
-        print "Warning: imaginary part of x ignored."
-        x = real(x)
+
     Xf = fft(x, N, axis=axis)
     h = zeros(N)
     if N % 2 == 0:
-        h[0] = h[N/2] = 1
-        h[1:N/2] = 2
+        h[0] = h[N / 2] = 1
+        h[1:N / 2] = 2
     else:
         h[0] = 1
-        h[1:(N+1)/2] = 2
+        h[1:(N + 1) / 2] = 2
 
     if len(x.shape) > 1:
-        ind = [newaxis]*x.ndim
+        ind = [newaxis] * x.ndim
         ind[axis] = slice(None)
         h = h[ind]
-    x = ifft(Xf*h, axis=axis)
+    x = ifft(Xf * h, axis=axis)
     return x
 
+
 def hilbert2(x, N=None):
     """
     Compute the '2-D' analytic signal of `x`
 
-
     Parameters
     ----------
     x : array_like
         2-D signal data.
-    N : int, optional
+    N : int or tuple of two ints, optional
         Number of Fourier components. Default is ``x.shape``
 
     Returns
@@ -717,37 +732,41 @@ def hilbert2(x, N=None):
         http://en.wikipedia.org/wiki/Analytic_signal
 
     """
-    x = asarray(x)
-    x = asarray(x)
+    x = atleast_2d(x)
+    if len(x.shape) > 2:
+        raise ValueError("x must be rank 2.")
+    if iscomplexobj(x):
+        raise ValueError("x must be real.")
     if N is None:
         N = x.shape
-    if len(N) < 2:
-        if N <=0:
+    elif isinstance(N, int):
+        if N <= 0:
             raise ValueError("N must be positive.")
-        N = (N,N)
-    if iscomplexobj(x):
-        print "Warning: imaginary part of x ignored."
-        x = real(x)
-    Xf = fft2(x,N,axes=(0,1))
-    h1 = zeros(N[0],'d')
-    h2 = zeros(N[1],'d')
+        N = (N, N)
+    elif len(N) != 2 or np.any(np.asarray(N) <= 0):
+        raise ValueError("When given as a tuple, N must hold exactly "
+                         "two positive integers")
+
+    Xf = fft2(x, N, axes=(0, 1))
+    h1 = zeros(N[0], 'd')
+    h2 = zeros(N[1], 'd')
     for p in range(2):
-        h = eval("h%d"%(p+1))
+        h = eval("h%d" % (p + 1))
         N1 = N[p]
         if N1 % 2 == 0:
-            h[0] = h[N1/2] = 1
-            h[1:N1/2] = 2
+            h[0] = h[N1 / 2] = 1
+            h[1:N1 / 2] = 2
         else:
             h[0] = 1
-            h[1:(N1+1)/2] = 2
-        exec("h%d = h" % (p+1), globals(), locals())
+            h[1:(N1 + 1) / 2] = 2
+        exec("h%d = h" % (p + 1), globals(), locals())
 
-    h = h1[:,newaxis] * h2[newaxis,:]
+    h = h1[:, newaxis] * h2[newaxis, :]
     k = len(x.shape)
     while k > 2:
         h = h[:, newaxis]
         k -= 1
-    x = ifft2(Xf*h,axes=(0,1))
+    x = ifft2(Xf * h, axes=(0, 1))
     return x
 
 
@@ -758,7 +777,8 @@ def cmplx_sort(p):
         indx = argsort(abs(p))
     else:
         indx = argsort(p)
-    return take(p,indx,0), indx
+    return take(p, indx, 0), indx
+
 
 def unique_roots(p, tol=1e-3, rtype='min'):
     """
@@ -802,23 +822,23 @@ def unique_roots(p, tol=1e-3, rtype='min'):
     array([ 1.305])
 
     """
-    if rtype in ['max','maximum']:
+    if rtype in ['max', 'maximum']:
         comproot = np.maximum
-    elif rtype in ['min','minimum']:
+    elif rtype in ['min', 'minimum']:
         comproot = np.minimum
-    elif rtype in ['avg','mean']:
+    elif rtype in ['avg', 'mean']:
         comproot = np.mean
-    p = asarray(p)*1.0
+    p = asarray(p) * 1.0
     tol = abs(tol)
     p, indx = cmplx_sort(p)
     pout = []
     mult = []
     indx = -1
-    curp = p[0] + 5*tol
+    curp = p[0] + 5 * tol
     sameroots = []
     for k in range(len(p)):
         tr = p[k]
-        if abs(tr-curp) < tol:
+        if abs(tr - curp) < tol:
             sameroots.append(tr)
             curp = comproot(sameroots)
             pout[indx] = curp
@@ -859,14 +879,14 @@ def invres(r, p, k, tol=1e-3, rtype='avg'):
     """
     extra = k
     p, indx = cmplx_sort(p)
-    r = take(r,indx,0)
-    pout, mult = unique_roots(p,tol=tol,rtype=rtype)
+    r = take(r, indx, 0)
+    pout, mult = unique_roots(p, tol=tol, rtype=rtype)
     p = []
     for k in range(len(pout)):
-        p.extend([pout[k]]*mult[k])
+        p.extend([pout[k]] * mult[k])
     a = atleast_1d(poly(p))
     if len(extra) > 0:
-        b = polymul(extra,a)
+        b = polymul(extra, a)
     else:
         b = [0]
     indx = 0
@@ -874,17 +894,18 @@ def invres(r, p, k, tol=1e-3, rtype='avg'):
         temp = []
         for l in range(len(pout)):
             if l != k:
-                temp.extend([pout[l]]*mult[l])
+                temp.extend([pout[l]] * mult[l])
         for m in range(mult[k]):
             t2 = temp[:]
-            t2.extend([pout[k]]*(mult[k]-m-1))
-            b = polyadd(b,r[indx]*poly(t2))
+            t2.extend([pout[k]] * (mult[k] - m - 1))
+            b = polyadd(b, r[indx] * poly(t2))
             indx += 1
     b = real_if_close(b)
     while allclose(b[0], 0, rtol=1e-14) and (b.shape[-1] > 1):
         b = b[1:]
     return b, a
 
+
 def residue(b, a, tol=1e-3, rtype='avg'):
     """
     Compute partial-fraction expansion of b(s) / a(s).
@@ -922,15 +943,15 @@ def residue(b, a, tol=1e-3, rtype='avg'):
 
     """
 
-    b,a = map(asarray,(b,a))
+    b, a = map(asarray, (b, a))
     rscale = a[0]
-    k,b = polydiv(b,a)
+    k, b = polydiv(b, a)
     p = roots(a)
-    r = p*0.0
-    pout, mult = unique_roots(p,tol=tol,rtype=rtype)
+    r = p * 0.0
+    pout, mult = unique_roots(p, tol=tol, rtype=rtype)
     p = []
     for n in range(len(pout)):
-        p.extend([pout[n]]*mult[n])
+        p.extend([pout[n]] * mult[n])
     p = asarray(p)
     # Compute the residue from the general formula
     indx = 0
@@ -939,22 +960,23 @@ def residue(b, a, tol=1e-3, rtype='avg'):
         pn = []
         for l in range(len(pout)):
             if l != n:
-                pn.extend([pout[l]]*mult[l])
+                pn.extend([pout[l]] * mult[l])
         an = atleast_1d(poly(pn))
         # bn(s) / an(s) is (s-po[n])**Nn * b(s) / a(s) where Nn is
         # multiplicity of pole at po[n]
         sig = mult[n]
-        for m in range(sig,0,-1):
+        for m in range(sig, 0, -1):
             if sig > m:
                 # compute next derivative of bn(s) / an(s)
-                term1 = polymul(polyder(bn,1),an)
-                term2 = polymul(bn,polyder(an,1))
-                bn = polysub(term1,term2)
-                an = polymul(an,an)
-            r[indx+m-1] = polyval(bn,pout[n]) / polyval(an,pout[n]) \
-                          / factorial(sig-m)
+                term1 = polymul(polyder(bn, 1), an)
+                term2 = polymul(bn, polyder(an, 1))
+                bn = polysub(term1, term2)
+                an = polymul(an, an)
+            r[indx + m - 1] = polyval(bn, pout[n]) / polyval(an, pout[n]) \
+                          / factorial(sig - m)
         indx += sig
-    return r/rscale, p, k
+    return r / rscale, p, k
+
 
 def residuez(b, a, tol=1e-3, rtype='avg'):
     """Compute partial-fraction expansion of b(z) / a(z).
@@ -981,21 +1003,21 @@ def residuez(b, a, tol=1e-3, rtype='avg'):
     invresz, poly, polyval, unique_roots
 
     """
-    b,a = map(asarray,(b,a))
+    b, a = map(asarray, (b, a))
     gain = a[0]
-    brev, arev = b[::-1],a[::-1]
-    krev,brev = polydiv(brev,arev)
+    brev, arev = b[::-1], a[::-1]
+    krev, brev = polydiv(brev, arev)
     if krev == []:
         k = []
     else:
         k = krev[::-1]
     b = brev[::-1]
     p = roots(a)
-    r = p*0.0
-    pout, mult = unique_roots(p,tol=tol,rtype=rtype)
+    r = p * 0.0
+    pout, mult = unique_roots(p, tol=tol, rtype=rtype)
     p = []
     for n in range(len(pout)):
-        p.extend([pout[n]]*mult[n])
+        p.extend([pout[n]] * mult[n])
     p = asarray(p)
     # Compute the residue from the general formula (for discrete-time)
     #  the polynomial is in z**(-1) and the multiplication is by terms
@@ -1007,22 +1029,24 @@ def residuez(b, a, tol=1e-3, rtype='avg'):
         pn = []
         for l in range(len(pout)):
             if l != n:
-                pn.extend([pout[l]]*mult[l])
+                pn.extend([pout[l]] * mult[l])
         an = atleast_1d(poly(pn))[::-1]
         # bn(z) / an(z) is (1-po[n] z**(-1))**Nn * b(z) / a(z) where Nn is
         # multiplicity of pole at po[n] and b(z) and a(z) are polynomials.
         sig = mult[n]
-        for m in range(sig,0,-1):
+        for m in range(sig, 0, -1):
             if sig > m:
                 # compute next derivative of bn(s) / an(s)
-                term1 = polymul(polyder(bn,1),an)
-                term2 = polymul(bn,polyder(an,1))
-                bn = polysub(term1,term2)
-                an = polymul(an,an)
-            r[indx+m-1] = polyval(bn,1.0/pout[n]) / polyval(an,1.0/pout[n]) \
-                          / factorial(sig-m) / (-pout[n])**(sig-m)
+                term1 = polymul(polyder(bn, 1), an)
+                term2 = polymul(bn, polyder(an, 1))
+                bn = polysub(term1, term2)
+                an = polymul(an, an)
+            r[indx + m - 1] = (polyval(bn, 1.0 / pout[n]) /
+                               polyval(an, 1.0 / pout[n]) /
+                               factorial(sig - m) / (-pout[n]) ** (sig - m))
         indx += sig
-    return r/gain, p, k
+    return r / gain, p, k
+
 
 def invresz(r, p, k, tol=1e-3, rtype='avg'):
     """Compute b(z) and a(z) from partial fraction expansion: r,p,k
@@ -1051,14 +1075,14 @@ def invresz(r, p, k, tol=1e-3, rtype='avg'):
     """
     extra = asarray(k)
     p, indx = cmplx_sort(p)
-    r = take(r,indx,0)
-    pout, mult = unique_roots(p,tol=tol,rtype=rtype)
+    r = take(r, indx, 0)
+    pout, mult = unique_roots(p, tol=tol, rtype=rtype)
     p = []
     for k in range(len(pout)):
-        p.extend([pout[k]]*mult[k])
+        p.extend([pout[k]] * mult[k])
     a = atleast_1d(poly(p))
     if len(extra) > 0:
-        b = polymul(extra,a)
+        b = polymul(extra, a)
     else:
         b = [0]
     indx = 0
@@ -1068,11 +1092,11 @@ def invresz(r, p, k, tol=1e-3, rtype='avg'):
         # Construct polynomial which does not include any of this root
         for l in range(len(pout)):
             if l != k:
-                temp.extend([pout[l]]*mult[l])
+                temp.extend([pout[l]] * mult[l])
         for m in range(mult[k]):
             t2 = temp[:]
-            t2.extend([pout[k]]*(mult[k]-m-1))
-            brev = polyadd(brev,(r[indx]*poly(t2))[::-1])
+            t2.extend([pout[k]] * (mult[k] - m - 1))
+            brev = polyadd(brev, (r[indx] * poly(t2))[::-1])
             indx += 1
     b = real_if_close(brev[::-1])
     return b, a
@@ -1137,7 +1161,7 @@ def resample(x, num, t=None, axis=0, window=None):
 
     """
     x = asarray(x)
-    X = fft(x,axis=axis)
+    X = fft(x, axis=axis)
     Nx = x.shape[axis]
     if window is not None:
         if callable(window):
@@ -1145,31 +1169,32 @@ def resample(x, num, t=None, axis=0, window=None):
         elif isinstance(window, ndarray) and window.shape == (Nx,):
             W = window
         else:
-            W = ifftshift(get_window(window,Nx))
+            W = ifftshift(get_window(window, Nx))
         newshape = ones(len(x.shape))
         newshape[axis] = len(W)
         W.shape = newshape
-        X = X*W
-    sl = [slice(None)]*len(x.shape)
+        X = X * W
+    sl = [slice(None)] * len(x.shape)
     newshape = list(x.shape)
     newshape[axis] = num
-    N = int(np.minimum(num,Nx))
-    Y = zeros(newshape,'D')
-    sl[axis] = slice(0,(N+1)/2)
+    N = int(np.minimum(num, Nx))
+    Y = zeros(newshape, 'D')
+    sl[axis] = slice(0, (N + 1) / 2)
     Y[sl] = X[sl]
-    sl[axis] = slice(-(N-1)/2,None)
+    sl[axis] = slice(-(N - 1) / 2, None)
     Y[sl] = X[sl]
-    y = ifft(Y,axis=axis)*(float(num)/float(Nx))
+    y = ifft(Y, axis=axis) * (float(num) / float(Nx))
 
-    if x.dtype.char not in ['F','D']:
+    if x.dtype.char not in ['F', 'D']:
         y = y.real
 
     if t is None:
         return y
     else:
-        new_t = arange(0,num)*(t[1]-t[0])* Nx / float(num) + t[0]
+        new_t = arange(0, num) * (t[1] - t[0]) * Nx / float(num) + t[0]
         return y, new_t
 
+
 def detrend(data, axis=-1, type='linear', bp=0):
     """
     Remove linear trend along axis from data.
@@ -1206,117 +1231,331 @@ def detrend(data, axis=-1, type='linear', bp=0):
     True
 
     """
-    if type not in ['linear','l','constant','c']:
+    if type not in ['linear', 'l', 'constant', 'c']:
         raise ValueError("Trend type must be 'linear' or 'constant'.")
     data = asarray(data)
     dtype = data.dtype.char
     if dtype not in 'dfDF':
         dtype = 'd'
-    if type in ['constant','c']:
-        ret = data - expand_dims(mean(data,axis),axis)
+    if type in ['constant', 'c']:
+        ret = data - expand_dims(mean(data, axis), axis)
         return ret
     else:
         dshape = data.shape
         N = dshape[axis]
-        bp = sort(unique(r_[0,bp,N]))
-        if any(bp > N):
+        bp = sort(unique(r_[0, bp, N]))
+        if np.any(bp > N):
             raise ValueError("Breakpoints must be less than length "
                     "of data along given axis.")
         Nreg = len(bp) - 1
         # Restructure data so that axis is along first dimension and
         #  all other dimensions are collapsed into second dimension
         rnk = len(dshape)
-        if axis < 0: axis = axis + rnk
-        newdims = r_[axis,0:axis,axis+1:rnk]
+        if axis < 0:
+            axis = axis + rnk
+        newdims = r_[axis, 0:axis, axis + 1:rnk]
         newdata = reshape(transpose(data, tuple(newdims)),
-                          (N, prod(dshape, axis=0)/N))
+                          (N, prod(dshape, axis=0) / N))
         newdata = newdata.copy()  # make sure we have a copy
         if newdata.dtype.char not in 'dfDF':
             newdata = newdata.astype(dtype)
         # Find leastsq fit and remove it for each piece
         for m in range(Nreg):
-            Npts = bp[m+1] - bp[m]
-            A = ones((Npts,2),dtype)
-            A[:,0] = cast[dtype](arange(1,Npts+1)*1.0/Npts)
-            sl = slice(bp[m],bp[m+1])
-            coef,resids,rank,s = linalg.lstsq(A,newdata[sl])
-            newdata[sl] = newdata[sl] - dot(A,coef)
+            Npts = bp[m + 1] - bp[m]
+            A = ones((Npts, 2), dtype)
+            A[:, 0] = cast[dtype](arange(1, Npts + 1) * 1.0 / Npts)
+            sl = slice(bp[m], bp[m + 1])
+            coef, resids, rank, s = linalg.lstsq(A, newdata[sl])
+            newdata[sl] = newdata[sl] - dot(A, coef)
         # Put data back in original shape.
-        tdshape = take(dshape,newdims,0)
-        ret = reshape(newdata,tuple(tdshape))
-        vals = range(1,rnk)
+        tdshape = take(dshape, newdims, 0)
+        ret = reshape(newdata, tuple(tdshape))
+        vals = range(1, rnk)
         olddims = vals[:axis] + [0] + vals[axis:]
-        ret = transpose(ret,tuple(olddims))
+        ret = transpose(ret, tuple(olddims))
         return ret
 
+
 def lfilter_zi(b, a):
-    #compute the zi state from the filter parameters. see [Gust96].
+    """
+    Compute an initial state `zi` for the lfilter function that corresponds
+    to the steady state of the step response.
 
-    #Based on:
-    # [Gust96] Fredrik Gustafsson, Determining the initial states in
-    #          forward-backward filtering, IEEE Transactions on
-    #          Signal Processing, pp. 988--992, April 1996,
-    #          Volume 44, Issue 4
+    A typical use of this function is to set the initial state so that the
+    output of the filter starts at the same value as the first element of
+    the signal to be filtered.
 
-    n=max(len(a),len(b))
+    Parameters
+    ----------
+    b, a : array_like (1-D)
+        The IIR filter coefficients. See `scipy.signal.lfilter` for more
+        information.
 
-    zin = (np.eye(n-1) - np.hstack((-a[1:n,newaxis],
-                                    np.vstack((np.eye(n-2),zeros(n-2))))))
+    Returns
+    -------
+    zi : 1-D ndarray
+        The initial state for the filter.
 
-    zid=  b[1:n] - a[1:n]*b[0]
+    Notes
+    -----
+    A linear filter with order m has a state space representation (A, B, C, D),
+    for which the output y of the filter can be expressed as::
 
-    zi_matrix=linalg.inv(zin)*(np.matrix(zid).transpose())
-    zi_return=[]
+        z(n+1) = A*z(n) + B*x(n)
+        y(n)   = C*z(n) + D*x(n)
 
-    #convert the result into a regular array (not a matrix)
-    for i in range(len(zi_matrix)):
-        zi_return.append(float(zi_matrix[i][0]))
+    where z(n) is a vector of length m, A has shape (m, m), B has shape
+    (m, 1), C has shape (1, m) and D has shape (1, 1) (assuming x(n) is
+    a scalar).  lfilter_zi solves::
 
-    return array(zi_return)
+        zi = A*zi + B
 
-def filtfilt(b, a, x):
-    b, a, x = map(asarray, [b, a, x])
-    # FIXME:  For now only accepting 1d arrays
-    ntaps=max(len(a),len(b))
-    edge=ntaps*3
+    In other words, it finds the initial condition for which the response
+    to an input of all ones is a constant.
 
-    if x.ndim != 1:
-        raise ValueError("filtfilt only accepts 1-d arrays.")
+    Given the filter coefficients `a` and `b`, the state space matrices
+    for the transposed direct form II implementation of the linear filter,
+    which is the implementation used by scipy.signal.lfilter, are::
 
-    #x must be bigger than edge
-    if x.size < edge:
-        raise ValueError("Input vector needs to be bigger than "
-              "3 * max(len(a),len(b).")
+        A = scipy.linalg.companion(a).T
+        B = b[1:] - a[1:]*b[0]
 
-    if len(a) < ntaps:
-        a=r_[a,zeros(len(b)-len(a))]
+    assuming `a[0]` is 1.0; if `a[0]` is not 1, `a` and `b` are first
+    divided by a[0].
 
-    if len(b) < ntaps:
-        b=r_[b,zeros(len(a)-len(b))]
+    Examples
+    --------
+    The following code creates a lowpass Butterworth filter. Then it
+    applies that filter to an array whose values are all 1.0; the
+    output is also all 1.0, as expected for a lowpass filter.  If the
+    `zi` argument of `lfilter` had not been given, the output would have
+    shown the transient signal.
 
-    zi = lfilter_zi(b,a)
+    >>> from numpy import array, ones
+    >>> from scipy.signal import lfilter, lfilter_zi, butter
+    >>> b, a = butter(5, 0.25)
+    >>> zi = lfilter_zi(b, a)
+    >>> y, zo = lfilter(b, a, ones(10), zi=zi)
+    >>> y
+    array([1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.])
 
-    #Grow the signal to have edges for stabilizing
-    #the filter with inverted replicas of the signal
-    s=r_[2*x[0]-x[edge:1:-1],x,2*x[-1]-x[-1:-edge:-1]]
-    #in the case of one go we only need one of the extrems
-    # both are needed for filtfilt
+    Another example:
 
-    (y,zf)=lfilter(b,a,s,-1,zi*s[0])
+    >>> x = array([0.5, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0])
+    >>> y, zf = lfilter(b, a, x, zi=zi*x[0])
+    >>> y
+    array([ 0.5       ,  0.5       ,  0.5       ,  0.49836039,  0.48610528,
+        0.44399389,  0.35505241])
 
-    (y,zf)=lfilter(b,a,flipud(y),-1,zi*y[-1])
+    Note that the `zi` argument to `lfilter` was computed using
+    `lfilter_zi` and scaled by `x[0]`.  Then the output `y` has no
+    transient until the input drops from 0.5 to 0.0.
 
-    return flipud(y[edge-1:-edge+1])
+    """
+
+    # FIXME: Can this function be replaced with an appropriate
+    # use of lfiltic?  For example, when b,a = butter(N,Wn),
+    #    lfiltic(b, a, y=numpy.ones_like(a), x=numpy.ones_like(b)).
+    #
+
+    # We could use scipy.signal.normalize, but it uses warnings in
+    # cases where a ValueError is more appropriate, and it allows
+    # b to be 2D.
+    b = np.atleast_1d(b)
+    if b.ndim != 1:
+        raise ValueError("Numerator b must be rank 1.")
+    a = np.atleast_1d(a)
+    if a.ndim != 1:
+        raise ValueError("Denominator a must be rank 1.")
+
+    while len(a) > 1 and a[0] == 0.0:
+        a = a[1:]
+    if a.size < 1:
+        raise ValueError("There must be at least one nonzero `a` coefficient.")
+
+    if a[0] != 1.0:
+        # Normalize the coefficients so a[0] == 1.
+        a = a / a[0]
+        b = b / a[0]
+
+    n = max(len(a), len(b))
+
+    # Pad a or b with zeros so they are the same length.
+    if len(a) < n:
+        a = np.r_[a, np.zeros(n - len(a))]
+    elif len(b) < n:
+        b = np.r_[b, np.zeros(n - len(b))]
+
+    IminusA = np.eye(n - 1) - linalg.companion(a).T
+    B = b[1:] - a[1:] * b[0]
+    # Solve zi = A*zi + B
+    zi = np.linalg.solve(IminusA, B)
+
+    # For future reference: we could also use the following
+    # explicit formulas to solve the linear system:
+    #
+    # zi = np.zeros(n - 1)
+    # zi[0] = B.sum() / IminusA[:,0].sum()
+    # asum = 1.0
+    # csum = 0.0
+    # for k in range(1,n-1):
+    #     asum += a[k]
+    #     csum += b[k] - a[k]*b[0]
+    #     zi[k] = asum*zi[0] - csum
+
+    return zi
+
+
+def filtfilt(b, a, x, axis=-1, padtype='odd', padlen=None):
+    """A forward-backward filter.
+
+    This function applies a linear filter twice, once forward
+    and once backwards.  The combined filter has linear phase.
+
+    Before applying the filter, the function can pad the data along the
+    given axis in one of three ways: odd, even or constant.  The odd
+    and even extensions have the corresponding symmetry about the end point
+    of the data.  The constant extension extends the data with the values
+    at end points.  On both the forward and backwards passes, the
+    initial condition of the filter is found by using lfilter_zi and
+    scaling it by the end point of the extended data.
+
+    Parameters
+    ----------
+    b : array_like, 1-D
+        The numerator coefficient vector of the filter.
+    a : array_like, 1-D
+        The denominator coefficient vector of the filter.  If a[0]
+        is not 1, then both a and b are normalized by a[0].
+    x : array_like
+        The array of data to be filtered.
+    axis : int, optional
+        The axis of `x` to which the filter is applied.
+        Default is -1.
+    padtype : str or None, optional
+        Must be 'odd', 'even', 'constant', or None.  This determines the
+        type of extension to use for the padded signal to which the filter
+        is applied.  If `padtype` is None, no padding is used.  The default
+        is 'odd'.
+    padlen : int or None, optional
+        The number of elements by which to extend `x` at both ends of
+        `axis` before applying the filter. This value must be less than
+        `x.shape[axis]-1`.  `padlen=0` implies no padding.
+        The default value is 3*max(len(a),len(b)).
+
+    Returns
+    -------
+    y : ndarray
+        The filtered output, an array of type numpy.float64 with the same
+        shape as `x`.
+
+    See Also
+    --------
+    lfilter_zi
+    lfilter
+
+    Examples
+    --------
+    First we create a one second signal that is the sum of two pure sine
+    waves, with frequencies 5 Hz and 250 Hz, sampled at 2000 Hz.
+
+    >>> t = np.linspace(0, 1.0, 2001)
+    >>> xlow = np.sin(2 * np.pi * 5 * t)
+    >>> xhigh = np.sin(2 * np.pi * 250 * t)
+    >>> x = xlow + xhigh
+
+    Now create a lowpass Butterworth filter with a cutoff of 0.125 times
+    the Nyquist rate, or 125 Hz, and apply it to x with filtfilt.  The
+    result should be approximately xlow, with no phase shift.
+
+    >>> from scipy.signal import butter
+    >>> b, a = butter(8, 0.125)
+    >>> y = filtfilt(b, a, x, padlen=150)
+    >>> np.abs(y - xlow).max()
+    9.1086182074789912e-06
+
+    We get a fairly clean result for this artificial example because
+    the odd extension is exact, and with the moderately long padding,
+    the filter's transients have dissipated by the time the actual data
+    is reached.  In general, transient effects at the edges are
+    unavoidable.
+    """
+
+    if padtype not in ['even', 'odd', 'constant', None]:
+        raise ValueError(("Unknown value '%s' given to padtype.  padtype must "
+                         "be 'even', 'odd', 'constant', or None.") %
+                            padtype)
+
+    b = np.asarray(b)
+    a = np.asarray(a)
+    x = np.asarray(x)
+
+    ntaps = max(len(a), len(b))
+
+    if padtype is None:
+        padlen = 0
+
+    if padlen is None:
+        # Original padding; preserved for backwards compatibility.
+        edge = ntaps * 3
+    else:
+        edge = padlen
+
+    # x's 'axis' dimension must be bigger than edge.
+    if x.shape[axis] <= edge:
+        raise ValueError("The length of the input vector x must be at least "
+                         "padlen, which is %d." % edge)
+
+    if padtype is not None and edge > 0:
+        # Make an extension of length `edge` at each
+        # end of the input array.
+        if padtype == 'even':
+            ext = even_ext(x, edge, axis=axis)
+        elif padtype == 'odd':
+            ext = odd_ext(x, edge, axis=axis)
+        else:
+            ext = const_ext(x, edge, axis=axis)
+    else:
+        ext = x
+
+    # Get the steady state of the filter's step response.
+    zi = lfilter_zi(b, a)
+
+    # Reshape zi and create x0 so that zi*x0 broadcasts
+    # to the correct value for the 'zi' keyword argument
+    # to lfilter.
+    zi_shape = [1] * x.ndim
+    zi_shape[axis] = zi.size
+    zi = np.reshape(zi, zi_shape)
+    x0 = axis_slice(ext, stop=1, axis=axis)
+
+    # Forward filter.
+    (y, zf) = lfilter(b, a, ext, zi=zi * x0)
+
+    # Backward filter.
+    # Create y0 so zi*y0 broadcasts appropriately.
+    y0 = axis_slice(y, start=-1, axis=axis)
+    (y, zf) = lfilter(b, a, axis_reverse(y, axis=axis), zi=zi * y0)
+
+    # Reverse y.
+    y = axis_reverse(y, axis=axis)
+
+    if edge > 0:
+        # Slice the actual signal from the extended signal.
+        y = axis_slice(y, start=edge, stop=-edge, axis=axis)
+
+    return y
 
 
 from scipy.signal.filter_design import cheby1
 from scipy.signal.fir_filter_design import firwin
 
-def decimate(x, q, n=None, ftype='iir', axis=-1):
-    """downsample the signal x by an integer factor q, using an order n filter
 
-    By default an order 8 Chebyshev type I filter is used or a 30 point FIR
-    filter with hamming window if ftype is 'fir'.
+def decimate(x, q, n=None, ftype='iir', axis=-1):
+    """Downsample the signal x by an integer factor q, using an order n filter.
+
+    By default an order 8 Chebyshev type I filter is used.  A 30 point FIR
+    filter with hamming window is used if ftype is 'fir'.
 
     Parameters
     ----------
@@ -1351,13 +1590,13 @@ def decimate(x, q, n=None, ftype='iir', axis=-1):
             n = 8
 
     if ftype == 'fir':
-        b = firwin(n+1, 1./q, window='hamming')
+        b = firwin(n + 1, 1. / q, window='hamming')
         a = 1.
     else:
-        b, a = cheby1(n, 0.05, 0.8/q)
+        b, a = cheby1(n, 0.05, 0.8 / q)
 
     y = lfilter(b, a, x, axis=axis)
 
-    sl = [None]*y.ndim
+    sl = [slice(None)] * y.ndim
     sl[axis] = slice(None, None, q)
     return y[sl]
