@@ -40,10 +40,35 @@ class bsr_matrix(_cs_matrix):
             ``data[ indptr[i]: indptr[i+1] ]``.  If the shape parameter is not
             supplied, the matrix dimensions are inferred from the index arrays.
 
+    Attributes
+    ----------
+    dtype : dtype
+        Data type of the matrix
+    shape : 2-tuple
+        Shape of the matrix
+    ndim : int
+        Number of dimensions (this is always 2)
+    nnz
+        Number of nonzero elements
+    data
+        Data array of the matrix
+    indices
+        BSR format index array
+    indptr
+        BSR format index pointer array
+    blocksize
+        Block size of the matrix
+    has_sorted_indices
+        Whether indices are sorted
+
     Notes
     -----
 
-    Summary
+    Sparse matrices can be used in arithmetic operations: they support
+    addition, subtraction, multiplication, division, and matrix power.
+
+    Summary of BSR format:
+
         - The Block Compressed Row (BSR) format is very similar to the
           Compressed Sparse Row (CSR) format.  BSR is appropriate for
           sparse matrices with dense sub matrices like the last example
