@@ -9,7 +9,8 @@
 
 from numpy.testing import TestCase, rand, assert_, assert_equal, \
     assert_almost_equal, assert_array_almost_equal, assert_array_equal, \
-    assert_approx_equal, assert_raises, run_module_suite
+    assert_approx_equal, assert_raises, run_module_suite, \
+    assert_allclose, dec
 from numpy import array, arange, zeros, ravel, float32, float64, power
 import numpy as np
 import sys
@@ -338,79 +339,113 @@ class TestCorrPearsonr(TestCase):
         assert_equal(r, -1.0)
         assert_equal(prob, 0.0)
 
-def test_fisher_exact():
+class TestFisherExact(TestCase):
     """Some tests to show that fisher_exact() works correctly.
 
-    Testing the hypergeometric survival function against R's, showing that one
-    of them (probably Scipy's) is slightly defective (see the test with
-    significant=1).  This is probably because, in distributions.py, Scipy
-    uses 1.0 - cdf as the sf instead of calculating the sf more directly
-    for improved numerical accuracy.
+    Note that in SciPy 0.9.0 this was not working well for large numbers due to
+    inaccuracy of the hypergeom distribution (see #1218). Fixed now.
 
     Also note that R and Scipy have different argument formats for their
-    hypergeometric distrib functions.
+    hypergeometric distribution functions.
 
     R:
     > phyper(18999, 99000, 110000, 39000, lower.tail = FALSE)
     [1] 1.701815e-09
     """
-    fisher_exact = stats.fisher_exact
+    def test_basic(self):
+        fisher_exact = stats.fisher_exact
 
-    res = fisher_exact([[18000, 80000], [20000, 90000]])[1]
-    assert_approx_equal(res, 0.2751, significant=4)
-    res = fisher_exact([[14500, 20000], [30000, 40000]])[1]
-    assert_approx_equal(res, 0.01106, significant=4)
-    res = fisher_exact([[100, 2], [1000, 5]])[1]
-    assert_approx_equal(res, 0.1301, significant=4)
-    res = fisher_exact([[2, 7], [8, 2]])[1]
-    assert_approx_equal(res, 0.0230141, significant=6)
-    res = fisher_exact([[5, 1], [10, 10]])[1]
-    assert_approx_equal(res, 0.1973244, significant=6)
-    res = fisher_exact([[5, 15], [20, 20]])[1]
-    assert_approx_equal(res, 0.0958044, significant=6)
-    res = fisher_exact([[5, 16], [20, 25]])[1]
-    assert_approx_equal(res, 0.1725862, significant=6)
-    res = fisher_exact([[10, 5], [10, 1]])[1]
-    assert_approx_equal(res, 0.1973244, significant=6)
-    res = fisher_exact([[5, 0], [1, 4]])[1]
-    assert_approx_equal(res, 0.04761904, significant=6)
-    res = fisher_exact([[0, 1], [3, 2]])[1]
-    assert_approx_equal(res, 1.0)
-    res = fisher_exact([[0, 2], [6, 4]])[1]
-    assert_approx_equal(res, 0.4545454545)
-    res = fisher_exact([[2, 7], [8, 2]])
-    assert_approx_equal(res[1], 0.0230141, significant=6)
-    assert_approx_equal(res[0], 4.0 / 56)
+        res = fisher_exact([[14500, 20000], [30000, 40000]])[1]
+        assert_approx_equal(res, 0.01106, significant=4)
+        res = fisher_exact([[100, 2], [1000, 5]])[1]
+        assert_approx_equal(res, 0.1301, significant=4)
+        res = fisher_exact([[2, 7], [8, 2]])[1]
+        assert_approx_equal(res, 0.0230141, significant=6)
+        res = fisher_exact([[5, 1], [10, 10]])[1]
+        assert_approx_equal(res, 0.1973244, significant=6)
+        res = fisher_exact([[5, 15], [20, 20]])[1]
+        assert_approx_equal(res, 0.0958044, significant=6)
+        res = fisher_exact([[5, 16], [20, 25]])[1]
+        assert_approx_equal(res, 0.1725862, significant=6)
+        res = fisher_exact([[10, 5], [10, 1]])[1]
+        assert_approx_equal(res, 0.1973244, significant=6)
+        res = fisher_exact([[5, 0], [1, 4]])[1]
+        assert_approx_equal(res, 0.04761904, significant=6)
+        res = fisher_exact([[0, 1], [3, 2]])[1]
+        assert_approx_equal(res, 1.0)
+        res = fisher_exact([[0, 2], [6, 4]])[1]
+        assert_approx_equal(res, 0.4545454545)
+        res = fisher_exact([[2, 7], [8, 2]])
+        assert_approx_equal(res[1], 0.0230141, significant=6)
+        assert_approx_equal(res[0], 4.0 / 56)
 
-    # High tolerance due to survival function inaccuracy.
-    res = fisher_exact([[19000, 80000], [20000, 90000]])[1]
-    assert_approx_equal(res, 3.319e-9, significant=1)
+    def test_precise(self):
+        fisher_exact = stats.fisher_exact
 
-    # results from R
-    #
-    # R defines oddsratio differently (see Notes section of fisher_exact
-    # docstring), so those will not match.  We leave them in anyway, in
-    # case they will be useful later on. We test only the p-value.
-    tablist = [
-        ([[100, 2], [1000, 5]], (2.505583993422285e-001,  1.300759363430016e-001)),
-        ([[2, 7], [8, 2]], (8.586235135736206e-002,  2.301413756522114e-002)),
-        ([[5, 1], [10, 10]], (4.725646047336584e+000,  1.973244147157190e-001)),
-        ([[5, 15], [20, 20]], (3.394396617440852e-001,  9.580440012477637e-002)),
-        ([[5, 16], [20, 25]], (3.960558326183334e-001,  1.725864953812994e-001)),
-        ([[10, 5], [10, 1]], (2.116112781158483e-001,  1.973244147157190e-001)),
-        ([[10, 5], [10, 0]], (0.000000000000000e+000,  6.126482213438734e-002)),
-        ([[5, 0], [1, 4]], (np.inf,  4.761904761904762e-002)),
-        ([[0, 5], [1, 4]], (0.000000000000000e+000,  1.000000000000000e+000)),
-        ([[5, 1], [0, 4]], (np.inf,  4.761904761904758e-002)),
-        ([[0, 1], [3, 2]], (0.000000000000000e+000,  1.000000000000000e+000))
-        ]
-    for table, res_r in tablist:
-        res = fisher_exact(np.asarray(table))
-        np.testing.assert_almost_equal(res[1], res_r[1], decimal=11,
-                                       verbose=True)
+        # results from R
+        #
+        # R defines oddsratio differently (see Notes section of fisher_exact
+        # docstring), so those will not match.  We leave them in anyway, in
+        # case they will be useful later on. We test only the p-value.
+        tablist = [
+            ([[100, 2], [1000, 5]], (2.505583993422285e-001,  1.300759363430016e-001)),
+            ([[2, 7], [8, 2]], (8.586235135736206e-002,  2.301413756522114e-002)),
+            ([[5, 1], [10, 10]], (4.725646047336584e+000,  1.973244147157190e-001)),
+            ([[5, 15], [20, 20]], (3.394396617440852e-001,  9.580440012477637e-002)),
+            ([[5, 16], [20, 25]], (3.960558326183334e-001,  1.725864953812994e-001)),
+            ([[10, 5], [10, 1]], (2.116112781158483e-001,  1.973244147157190e-001)),
+            ([[10, 5], [10, 0]], (0.000000000000000e+000,  6.126482213438734e-002)),
+            ([[5, 0], [1, 4]], (np.inf,  4.761904761904762e-002)),
+            ([[0, 5], [1, 4]], (0.000000000000000e+000,  1.000000000000000e+000)),
+            ([[5, 1], [0, 4]], (np.inf,  4.761904761904758e-002)),
+            ([[0, 1], [3, 2]], (0.000000000000000e+000,  1.000000000000000e+000))
+            ]
+        for table, res_r in tablist:
+            res = stats.fisher_exact(np.asarray(table))
+            np.testing.assert_almost_equal(res[1], res_r[1], decimal=11,
+                                           verbose=True)
 
-    # test we raise an error for wrong shape of input.
-    assert_raises(ValueError, fisher_exact, np.arange(6).reshape(2, 3))
+    @dec.slow
+    def test_large_numbers(self):
+        # Test with some large numbers. Regression test for #1401
+        pvals = [5.56e-11, 2.666e-11, 1.363e-11]  # from R
+        for pval, num in zip(pvals, [75, 76, 77]):
+            res = stats.fisher_exact([[17704, 496], [1065, num]])[1]
+            assert_approx_equal(res, pval, significant=4)
+
+        res = stats.fisher_exact([[18000, 80000], [20000, 90000]])[1]
+        assert_approx_equal(res, 0.2751, significant=4)
+
+    def test_raises(self):
+        # test we raise an error for wrong shape of input.
+        assert_raises(ValueError, stats.fisher_exact,
+                      np.arange(6).reshape(2, 3))
+
+    def test_row_or_col_zero(self):
+        tables = ([[0, 0], [5, 10]],
+                  [[5, 10], [0, 0]],
+                  [[0, 5], [0, 10]],
+                  [[5, 0], [10, 0]])
+        for table in tables:
+            oddsratio, pval = stats.fisher_exact(table)
+            assert_equal(pval, 1.0)
+            assert_equal(oddsratio, np.nan)
+
+    def test_less_greater(self):
+        tables = ([[2, 7], [8, 2]],
+                  [[200, 7], [8, 300]],
+                  [[28, 21], [6, 1957]],
+                  [[190, 800], [200, 900]])
+        # from R
+        pvals = ([0.018521725952066501, 0.9990149169715733],
+                 [1.0, 2.0056578803889148e-122],
+                 [1.0, 5.7284374608319831e-44],
+                 [0.7416227, 0.2959826])
+        for table, pval in zip(tables, pvals):
+            res = []
+            res.append(stats.fisher_exact(table, alternative="less")[1])
+            res.append(stats.fisher_exact(table, alternative="greater")[1])
+            assert_allclose(res, pval, atol=0, rtol=1e-7)
 
 
 class TestCorrSpearmanr(TestCase):
@@ -559,8 +594,8 @@ def test_kendalltau():
     # with some ties
     x1 = [12, 2, 1, 12, 2]
     x2 = [1, 4, 7, 1, 0]
-    res = (-0.47140452079103173, 0.24821309157521476)
-    expected = stats.kendalltau(x1, x2)
+    expected = (-0.47140452079103173, 0.24821309157521476)
+    res = stats.kendalltau(x1, x2)
     assert_approx_equal(res[0], expected[0])
     assert_approx_equal(res[1], expected[1])
 
@@ -1488,6 +1523,13 @@ def test_normalitytests():
     yield assert_array_almost_equal, stats.skewtest(x), (st_skew, pv_skew)
     yield assert_array_almost_equal, stats.kurtosistest(x), (st_kurt, pv_kurt)
 
+def test_skewtest_too_few_samples():
+    """Regression test for ticket #1492.
+
+    skewtest requires at least 8 samples; 7 should raise a ValueError.
+    """
+    x = np.arange(7.0)
+    assert_raises(ValueError, stats.skewtest, x)
 
 def mannwhitneyu():
     x = np.array([ 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
