@@ -21,9 +21,9 @@ import sys
 import os
 import time
 import tempfile
-import exceptions
 import commands
 import subprocess
+import warnings
 
 import platform_info
 
@@ -72,7 +72,7 @@ distutils.sysconfig._init_posix = _init_posix
 # end force g++
 
 
-class CompileError(exceptions.Exception):
+class CompileError(Exception):
     pass
 
 
@@ -336,19 +336,19 @@ def choose_compiler(compiler_name=''):
             compiler_name = 'unix'
     return compiler_name
 
-def gcc_exists(name = 'gcc'):
-    """ Test to make sure gcc is found
-
-        Does this return correct value on win98???
-    """
+def gcc_exists(name='gcc'):
+    """ Test to make sure gcc is found."""
     result = 0
     cmd = [str(name), '-v']
     try:
-        p = subprocess.Popen(cmd, True, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT)
+        if sys.platform == 'win32':
+            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
+                                 stderr=subprocess.STDOUT)
+        else:
+            p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                                 stderr=subprocess.STDOUT)
         str_result = p.stdout.read()
-        #print str_result
-        if 'Reading specs' in str_result:
+        if 'specs' in str_result:
             result = 1
     except:
         # This was needed because the msvc compiler messes with
@@ -407,15 +407,16 @@ def configure_temp_dir(temp_dir=None):
     if not os.access(temp_dir,os.W_OK):
         msg = "Either the temp or build directory wasn't writable. Check" \
               " these locations: '%s'" % temp_dir
-        raise ValueError, msg
+        raise ValueError(msg)
     return temp_dir
 
 def configure_build_dir(build_dir=None):
     # make sure build_dir exists and is writable
     if build_dir and (not os.path.exists(build_dir) or
                       not os.access(build_dir,os.W_OK)):
-        print "warning: specified build_dir '%s' does not exist " \
-               "or is not writable. Trying default locations" % build_dir
+        msg = "specified build_dir '%s' does not exist " \
+              "or is not writable. Trying default locations" % build_dir
+        warnings.warn(msg)
         build_dir = None
 
     if build_dir is None:
@@ -433,7 +434,7 @@ def configure_build_dir(build_dir=None):
     if not os.access(build_dir,os.W_OK):
         msg = "The build directory wasn't writable. Check" \
               " this location: '%s'" % build_dir
-        raise ValueError, msg
+        raise ValueError(msg)
 
     return os.path.abspath(build_dir)
 
